package cn.iocoder.yudao.module.system.handler.mapnode;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.common.util.collection.CollectionUtils;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.dto.NodeBaseDTO;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapItemDO;
import cn.iocoder.yudao.module.system.handler.mapnode.strategy.HouseLocationStrategyImpl;
import cn.iocoder.yudao.module.system.handler.mapnode.strategy.NodeProcessingStrategy;
import cn.iocoder.yudao.module.system.service.positionmap.PositionMapItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.baomidou.mybatisplus.core.toolkit.IdWorker.getId;

@Component
public class NodeProcessingContext {

    private final Map<Integer, NodeProcessingStrategy> strategyMap = new HashMap<>();


    @Autowired
    private HouseLocationStrategyImpl houseLocationStrategyImpl;
    @Autowired
    private PositionMapItemService positionMapItemService;

    /**
     * 项目启动时 初始化策略
     */
    @PostConstruct
    public void init() {
        // 节点
//        strategyMap.put(1, positionMapItemStrategyImpl);
        // 库位
        strategyMap.put(2, houseLocationStrategyImpl);
        // todo 可以继续添加更多的策略
    }

    public void processNodesByStrategy(Long positionMapId, List<PositionMapItemDO> oldItemList, int key, List<NodeBaseDTO> nodeBaseDTOS) {
        // -- 这里处理节点 - 因为所有的类型都要建立在节点之上 - 把所有新增没有节点id的先分配id下 - 后续其他类型的需要绑定该id
        nodeBaseDTOS.forEach(a->{if (a.getId() == null) {a.setId(getId());}});
        List<PositionMapItemDO> newList = BeanUtil.copyToList(nodeBaseDTOS, PositionMapItemDO.class);
        List<List<PositionMapItemDO>> list = CollectionUtils.diffList(oldItemList, newList,
                (oldVal, newVal) -> ObjectUtil.equal(oldVal.getId(), newVal.getId()));
        positionMapItemService.batchSaveOrEditOrDel(positionMapId, list);
        // -- 获取对应的策略 - 如果没有对应的直接 return
        NodeProcessingStrategy strategy = strategyMap.get(key);
        if (strategy == null) {
            return;
        }
        strategy.processNodes(positionMapId, nodeBaseDTOS);
    }
}
